// -*- C++ -*-
//===-----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

#ifndef _LIBCPP___LOCALE_DIR_LOCALE_BASE_API_OPENBSD_H
#define _LIBCPP___LOCALE_DIR_LOCALE_BASE_API_OPENBSD_H

#include <__support/xlocale/__strtonum_fallback.h>
#include <clocale>
#include <cstdlib>
#include <ctype.h>
#include <cwctype>

#endif // _LIBCPP___LOCALE_DIR_LOCALE_BASE_API_OPENBSD_H
